require "game_object"

Bullet = class("Bullet", GameObject)


--[[
-- x, y - bullet coords
-- a - angle at which the bullet travels
-- i - bullet image
-- hc - instance of HardonCollider
-- v - speed
-- md - max travel distance
--]]
function Bullet:initialize(x, y, a, i, hc, v, md)
  GameObject.initialize(self, x, y, true, self.v, a, 0, i, hc)
  self.x = x
  self.y = y
  self.a = a
  self.i = i
  self.x_heading = math.cos(a)
  self.y_heading = math.sin(a)
  self.v = v
  self.w = self.i:getWidth()
  self.h = self.i:getHeight()
  self.ox = self.w / 2
  self.oy = self.h / 2
  self.max_distance = md
  self.distance_travelled = 0
  self.travelled_too_far = false
end

function Bullet:update(dt)

  local dx = self.v * dt * self.x_heading
  local dy = self.v * dt * self.y_heading
  self.x = self.x + dx
  self.y = self.y + dy
  self.distance_travelled= self.distance_travelled + math.sqrt(dx * dx + dy * dy)
  if self.distance_travelled > self.max_distance then
    self.travelled_too_far = true
  end
  
  GameObject.update(self, dt)
end
