class = require "lib.middleclass.middleclass"

--[[
-- Generate and draw a random (within reason) number of pixels starting at the same
-- location and exploding in random directions at random speed outwards from that location.
--]]
Explosion = class("Explosion")

--[[
-- x, y : starting coords
-- c : color of the particles
-- n : max number of particles. n/2 particles are guaranteed. The rest are random.
-- time: time period during which the explosion is drawn. After this time period, calls
--       to update do not update coords and calls to draw do not draw anything. At this
--       point, the explosion should be removed from the game loop.
--]]
function Explosion:initialize(x, y, c, n, time)
  self.c = c
  self.timer = time

  self.ps = {} -- particles
  local v = 0 -- velocity
  local a = 0 -- angle


  for i=1,n do
    if i <= n/2 or math.random() > 0.5 then
      v = math.random() * 100
      a = math.random() * 2 * math.pi
      table.insert(self.ps, {x = x, y = y, v = v, a = a})
    end
  end
end

function Explosion:update(dt)
  if self.timer > 0 then
    self.timer = self.timer - dt
  end

  for i, p in pairs(self.ps) do
    p.x = p.x + p.v * math.cos(p.a) * dt
    p.y = p.y + p.v * math.sin(p.a) * dt
  end

end

function Explosion:draw()
  if self.timer > 0 then
    local r, g, b, a = love.graphics.getColor()
    love.graphics.setColor(self.c)
    
    for i, p in pairs(self.ps) do
      love.graphics.points(p.x, p.y)
    end
    
    love.graphics.setColor(r, g, b, a)
  end
end
