class = require "lib.middleclass.middleclass"
require "highscorestore"
require "lib/SUIT"

HIGHSCORES_FILENAME = "highscores.txt"
HIGHSCORES_MAX = 10

GameOver = class("Gameover")

function GameOver:initialize()
  self.hss = HighScoreStore:new(HIGHSCORES_FILENAME, HIGHSCORES_MAX)
  self.hs = self.hss:get()
  -- Prompt for name if high scores are empty
  self.prompt_for_name = false
  self.name_input = {text = "", forcefocus = true}
  self.fade_time = 0
  self.fade_alpha = 0
end


function GameOver:setScore(score)
  self.score = score

  if #self.hs < HIGHSCORES_MAX or self.score >= self.hs[#self.hs].score then
    self.prompt_for_name = true
  end
end

function GameOver:update(dt)
  self.fade_time = self.fade_time + 4 * dt
  self.fade_alpha = 200 + (55 * math.sin(self.fade_time))

  -- use the SUIT GUI framework to let the player enter name and add it to the top 10 scores
  if self.prompt_for_name and not self.name_entered then
    local font = love.graphics.newFont("asset/font/Atmosphere-Light.TTF", 24)
    
    suit.Label("Welcome to top 10!", {font=font}, 210, 300, WINDOW_WIDTH - 420, 30)
    suit.Label("Score : " .. self.score, {font=font}, 210, 340,  WINDOW_WIDTH - 420, 30)
    
    local ret = suit.Input(self.name_input, {font=font, id=self.name_input_id}, 260, 380, WINDOW_WIDTH - 520, 40)
    
    -- Detect Enter/Return in the text box and add the score to the top list
    if ret.submitted == true then
      -- trim whitespace
      self.name_input.text = self.name_input.text:gsub("^%s*(.-)%s*$", "%1")
      -- add name to high scores
      self.hss:add(HighScore:new(self.name_input.text, self.score))
      -- re-read the high scores file so that name is displayed
      self.hs = self.hss:get()
      -- set prompt_for_name to false so that this dialog is not displayed
      self.prompt_for_name = false
    end
  end

end

function GameOver:draw()
  -- Store font, color and line width settings before changing them
  local default_font = love.graphics.getFont()
  local cr, cg, cb, ca = love.graphics.getColor()
  local lw = love.graphics.getLineWidth()

  -- Large "GAME OVER" at the top
  local font = love.graphics.newFont("asset/font/Atmosphere-Bold.TTF", 48)
  love.graphics.setColor(252, 0, 117)
  love.graphics.setFont(font)
  local str_w = font:getWidth("G A M E  O V E R")
  local str_h = font:getHeight()
  love.graphics.print("G A M E  O V E R", (WINDOW_WIDTH - str_w) / 2, 64)

  -- Smaller "HIGH SCORES" title
  font = love.graphics.newFont("asset/font/Atmosphere-Regular.TTF", 28)
  love.graphics.setColor(0, 172, 242)
  love.graphics.setFont(font)
  str_w = font:getWidth("H I G H  S C O R E S")
  str_h = font:getHeight()
  love.graphics.print("H I G H  S C O R E S", (WINDOW_WIDTH - str_w) / 2, 180)

  -- Smaller yet font for the high scores table
  font = love.graphics.newFont("asset/font/Atmosphere-Light.TTF", 24)
  love.graphics.setFont(font)

  -- Figure out spacing for the high score table.
  -- Each row consists of: <high score place>   <name>   <score>
  -- High score place and name are left aligned.
  -- Score is right aligned to make it look more centered on the screen.
  str_h = font:getHeight()
  local place_str_width = font:getWidth("10.")
  local name_str_width = font:getWidth("WWWWWWWWWWWWWWWWWWWWWWWW")
  local score_str_width = font:getWidth("0000000000")
  local column_pad = 10
  local max_row_width = place_str_width + column_pad + name_str_width + column_pad + score_str_width
  local row_pad = 4

  local place_x = (WINDOW_WIDTH - max_row_width) / 2
  local name_x = place_x + place_str_width + column_pad
  local score_x = name_x + name_str_width + column_pad

  for i=1, HIGHSCORES_MAX, 1 do
    local row_y = 200 + (str_h + row_pad) * i
    love.graphics.print(string.format("%d.", i), place_x, row_y)
    if self.hs[i] then
      love.graphics.print(string.format("%-24s", self.hs[i].name:sub(1,24)), name_x, row_y)
      love.graphics.print(string.format("%-10d", self.hs[i].score), max_row_width + (WINDOW_WIDTH - max_row_width) / 2 - font:getWidth(tostring(self.hs[i].score)), row_y)
    else
      love.graphics.print(". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .", name_x, row_y)
      love.graphics.print(string.format("%d", 0), max_row_width + (WINDOW_WIDTH - max_row_width) / 2 - font:getWidth("0"), row_y)
    end
  end

  -- If the player has a score in the top 10, ask for player's name.
  -- Draw the dialog box frame.
  if self.prompt_for_name then
    love.graphics.setColor(0, 50, 120, 200)
    love.graphics.rectangle("fill", 200, 280, WINDOW_WIDTH - 400, 170)
    love.graphics.setColor(0, 172, 242, 255)
    love.graphics.setLineWidth(3)
    love.graphics.rectangle("line", 200, 280, WINDOW_WIDTH - 400, 170)

    -- Let the GUI framework draw controls in the dialog box.
    suit.draw()
  end

  -- Fade "ESC to Exit" text in and out to draw attention to it
  love.graphics.setColor(255, 255, 255, self.fade_alpha)
  love.graphics.print("Press ESC to Exit", (WINDOW_WIDTH - font:getWidth("Press ESC to Exit")) / 2,  WINDOW_HEIGHT - 80)

  -- restore font, color, line settings back to what they were before this call
  love.graphics.setFont(default_font)
  love.graphics.setColor(cr, cg, cb, ca)
  love.graphics.setLineWidth(lw)
end
