class = require "lib.middleclass.middleclass"

--[[
-- Play a sound at specified rate.
-- Rate is specified in Hz.
--]]
HeartbeatAudio = class("HeartbeatAudio")

function HeartbeatAudio:initialize(sound)
  self.sound = sound
  self.prevRate = 0
  self.rate = 0
  self.timer = 0
end

function HeartbeatAudio:setRate(rate)
  self.prevRate = self.rate
  self.rate = rate
end

function HeartbeatAudio:play(rate)
  -- This is not a mistake, play() resets prev rate
  self.prevRate = rate
  self.rate = rate
end

function HeartbeatAudio:stop()
  self.prevRate = 0
  self.rate = 0
  self.timer = 0
end

function HeartbeatAudio:update(dt)
  local rate = 0
  if self.rate > 0 then
    self.timer = self.timer + dt
    if self.prevRate ~= self.rate then
      -- allow the previous rate to beat one last time to make
      -- the transition to the new rate audibly smooth
      rate = self.prevRate
    else
      rate = self.rate
    end

    if self.timer > 1 / rate then
      self.sound:play()
      self.timer = 0
      self.lastBeatTime = 0
      if self.prevRate ~= self.rate then
        self.prevRate = self.rate
      end
    end
  end
end

